from ngsolve import *
from netgen.geom2d import SplineGeometry

# ----------------- PARAMETERS -----------------
h_max = 0.082
k = 5
StatCond = True
inverse = "umfpack"

nu= 0.001
pReg = 1e-10

# ----------------- GEOMETRY & MESH -----------------
geo = SplineGeometry()
geo.AddRectangle((0,0),(2.2,0.41), bcs=("wall","outlet","wall","inlet"))
geo.AddCircle( (0.2, 0.2), r=0.05, leftdomain=0, rightdomain=1, bc="cyl")
mesh = Mesh(geo.GenerateMesh(maxh=h_max, quad_dominated=False))
mesh.Curve(k)

# ----------------- FE SPACE -----------------
V = VectorH1(mesh, order=k, dirichlet="wall|inlet|cyl")
Q = H1(mesh, order=k-1)
X = FESpace([V,Q])

gfu = GridFunction(X)

# ----------------- BILINEAR FORMS -----------------
(u,p), (v,q) = X.TnT()
n = specialcf.normal(mesh.dim)

Stokes = nu*InnerProduct(grad(u)+grad(u).trans,grad(v)) - p*div(v) - q*div(u) - pReg*p*q
TensorCorrect = -nu*InnerProduct((grad(u).trans)*n,v)

a = BilinearForm(X, symmetric=False, condense=StatCond)
a += SymbolicBFI(form = Stokes)
a += SymbolicBFI(form = TensorCorrect, VOL_or_BND=BND, skeleton=True, definedon=mesh.Boundaries("outlet"))
with TaskManager():
	a.Assemble()
	inv = a.mat.Inverse( X.FreeDofs(coupling=StatCond), inverse=inverse)

# ----------------- SOLVE PROBLEM -----------------
res = gfu.vec.CreateVector()
du = gfu.vec.CreateVector()

with TaskManager():

	# RHS Data
	gfu.components[0].Set(
		CoefficientFunction( (4*0.3*(y*(0.41-y)/(0.41**2)), 0.0 ) ),
		definedon=mesh.Boundaries("inlet")
		)
	res.data = -a.mat*gfu.vec
	if StatCond:
		res.data += a.harmonic_extension_trans * res

	# Solve problem
	du.data = inv * res	
	if StatCond:
		du.data += a.inner_solve * res
		du.data += a.harmonic_extension * du
		
	gfu.vec.data += du

Draw(gfu.components[0], mesh, "vel")
Draw(gfu.components[1], mesh, "pre")